/*  PhotonFinish2.v
  Started Jan 2012 by Alan Stummer, University of Toronto
  DESCRIPTION:  On an Arria II GX evaluation board, interfaces with the Acam IC via the HSMC-A to
                the rest of the FPGA and the rest of the world.  This is the top level (and
				currently the only) user HDL file.
  
  NOTES:  Refer to the protocols document for address mapping and other details.
  
  NOMENCLATURE:
    "Arria II GX Eval Board" and variants means as it says, a board from Altera with an Arria FPGA,
        USB, Ethernet, PCIe, LCD, pushbuttons, LEDs, etc.
    "FPGA" refers to the Altera EP2AGX125EF35C4 on the Arria eval board.
    "Acam" means the TD-GPX time-to-digital converter IC.  Refer to its manual for details.
    "HSMC" and variants means the High Speed Mezzanine Connector, both the PhotonFinish2 PCB with
        the Acam device and the connector which connects it to the Arria II GX eval board.
    "Nios II" is the embedded soft processor in the FPGA.
    "Avalon" is the bus used by Nios II.
*/


`timescale 1 ps / 1 ps

module AcamInterface (
	//Acam board I/Os via HSMC-A
	inout  [27:0] AcamData,
	output [ 3:0] AcamAddr,
	output        nAcamCS,
	output        AcamClk,
	output        AcamReset,
	output        nAcamWR,
	output        nAcamRD,
	output        nAcamOE,//   = 1,
	output        AcamStart,// = 0,															//currently not used.
	input         AcamEF1,
	input         AcamEF2,
	input         AcamLF1,
	input         AcamLF2,
	input         AcamIRQ,
	input         AcamErr,
	inout         HsmcIO1,						 //TTL in or out, selectable by switch on the Acam board.
	inout         HsmcIO2,						 //TTL in or out, selectable by switch on the Acam board.
	input  [ 3:1] HsmcOption,										//three option switches on the Acam board.
	//Arria II GX Eval Board I/Os
	input  [ 1:0] PushButton,
	output [ 3:0] Led,
	input         ClkIn,																			 //40MHz, for Avalon.
	input         Reset,
	//Internal Avalonian Bus
	output [31:0] AvalonDataRD,												//this module writes to this bus.
	input  [31:0] AvalonDataWR,											  //this module reads from this bus.
	input  [ 4:0] AvalonAddr,								 //0x00-0F for Acam, 0x10-1F for FPGA registers.
	input         AvalonCS,
	input         AvalonReadEN,
	input         AvalonWriteEN,
	//Ethernet Control
	input         EnetCtrlOut,															 //control input from Qsys.
	input         EnetCtrlOen,										 //true when Qsys is sending control data.
	output        EnetCtrlIn,															 //control oputput to Qsys.
	input         EnetCtrlMdc,															//control clock, from Qsys.
	//Ethernet Rx and Tx
	input  [31:0] EnetRxData,														//incoming data from the host.
	input         EnetRxValid,							  //flags as valid RxData[], RxSop, RxEop, RxErr[].
	input         EnetRxEop,									 //End Of Packet, true during last data word.
	input  [ 1:0] EnetRxEmpty,																		 //haven't a clue.
	output        EnetRxReady = 1,													//assert when ready to rcv.
	input         EnetRxSop,								 //Start Of Packet, true during first data word.
	input  [ 5:0] EnetRxError,										//error signals, [0] true if any are true.
	output [31:0] EnetTxData,												 //outgoing data, going to the host.
	output        EnetTxEop,									  //End Of Packet, set true during last word.
	output [ 1:0] EnetTxEmpty,						//not sure, probably how full or empty the Tx FIFO is.
	input         EnetTxReady,									//when true, MAC is ready for another packet.
	input         EnetRxClk,															  //Rx clock from MAC chip.
	output        EnetTxSop,								  //Start of Packet, set true during first word.
	output        EnetTxValid							//set true to indicate that following are valid:...
	);								 //...data_tx_data_n, data_tx_sop_n, data_tx_eop_n and data_tx_error_ n.


	//Parameters **********************
	localparam  VerilogVer = 24'h003;									  //update the Verilog version here.


	//Locals **************************
	reg  [31:0] FpgaReg = 32'h001C;		//Led[3] on				  //FPGA control register, set by Nios.
	wire [31:0] VerAndSwitches;		//word for Nios with Verilog version, pushbuttons and switches.
	reg  [24:0] Ctr;
	reg  [15:0] PacketCtr = 16'd0;						  //@@TEST TEST OF INCOMING MAC PACKET COUNTING.
	reg         InPacket = 0;					//@@ true when in a packet.

		
	//Combinational to pipe Avalon to Acam ********************************************************
	assign VerAndSwitches[23: 0] = VerilogVer;										 //Verilog version word.
	assign VerAndSwitches[25:24] = PushButton[1:0];					//Arria eval board user pushbuttons.
	assign VerAndSwitches[28:26] = HsmcOption[3:1];					  //HSMC Acam board option switches.
	assign VerAndSwitches[31:29] = 3'd0;
	assign AcamReset = FpgaReg[0];														//reset the Acam device.
	assign Led[0] = EnetRxSop;		//FpgaReg[2];
	assign Led[1] = EnetRxEop;		//FpgaReg[3];
	assign Led[2] = EnetRxValid;	//FpgaReg[4];
	assign Led[3] = Ctr[24];		//FpgaReg[5];
	assign AcamClk   = ClkIn;														  //pipe Avalon clock to Acam.
	assign nAcamCS   = !(AvalonCS && (AvalonAddr < 5'h10));					  //pipe Avalon CS to Acam.
	assign nAcamWR   = !(AvalonCS && AvalonWriteEN);							//pipe Avalon WRen to Acam.
	assign nAcamRD   = 1;											//currently, Avalon never reads from Acam.
	assign AcamAddr  = AvalonAddr[3:0];							  //pipe Avalon 4 LS address bits to Acam.
	assign AcamData  = (AvalonCS && AvalonWriteEN && AvalonAddr < 5'h10)	//when Avalon writes Acam..
	          ?  AvalonDataWR[27:0]										    //...pipe Avalon data to Acam...
		       :  28'bz;													  //...otherwise tristate the Acam bus.


	//Avalon (Nios) is Accessing the FPGA **********************************************************
	always @(posedge ClkIn)  begin
		if (AvalonCS)  begin
			if (AvalonWriteEN)  begin															//READ FROM THE ACAM.
				case (AvalonAddr)
					5'h10:  FpgaReg <= AvalonDataWR;					  //write to the FPGA control register.
					default:  ;
				endcase
			end

			if (AvalonReadEN)  begin															 //WRITE TO THE ACAM.
				case (AvalonAddr)
					5'h11:  AvalonDataRD <= VerAndSwitches;			//read Verilog version and switches.
					5'h12:  AvalonDataRD <= {13'd0, EnetRxSop, EnetRxEop, EnetRxValid, PacketCtr}; //@@ TEST e'net rx info.
					default:  ;
				endcase
			end
		end
	end


// Ethernet Receive, configured as 10/100 MII *****************************************************
	always @(posedge EnetRxClk)  begin	//100Mbaud is 25MHz
		if (EnetRxSop && !InPacket)  begin						//@@start of a packet.
			InPacket  <= 1;
			PacketCtr <= PacketCtr + 16'd1;
		end
		
		if (EnetRxEop && InPacket)  InPacket <= 0;			//@@end of a packet.
	end

	
	//Run Time, Read Acam Coincidences *************************************************************
	always @(posedge ClkIn)  begin
		Ctr <= Ctr + 25'd1;								//@@ blink LED3.
	//		if (FpgaReg[1] == 1)  begin							//RUN REALTIME DATA COLLECTION FROM ACAM.
			//insert runtime code here...
	//	end
	end

endmodule

